import React, { useEffect, useState } from 'react';
import {
  View,
  Image,
  getSystemInfoSync,
  connectBTBond,
  openSystemBluetoothSetting,
} from '@ray-js/ray';
import { useSelector, useDispatch } from 'react-redux';
import { useActions, useProps, useDevice } from '@ray-js/panel-sdk';
import Foo from '@ray-js/lamp-vertical-percent-slider';
import Popup from '@ray-js/components-ty-popup';
import _ from 'lodash-es';
import iconv from 'iconv-lite';
import { Buffer } from 'buffer';

import I18N from '@/i18n';
import styles from './index.module.less';
import BleState from './components/ble_state/index';
import Tabbar from './components/tabbar/index';
import Navbar from '@/components/navbar/index';

import {
  selectMusicList,
  selectFavorite,
  updatedMusicList,
  updatedFavorites,
  updatedMusicTag,
} from '@/redux/modules/uiStateSlice';
import { getStorage } from '@/utils/deviceProperty';

const cardTypes: {
  name: string;
  id: number;
}[] = [];
/**
 * cardTypes的下标
 */
let card_index = 0;

/**
 * 当前包数
 */
let cur_package_num = 1;
/**
 * 总包数
 */
let total_num = 0;
/**
 * 歌曲id
 */
let music_id = 0;

const isIos = getSystemInfoSync().platform === 'ios';

const Home = props => {
  const dpaction = useActions();
  const dispath = useDispatch();
  const bluetooth_volume = useProps(props => props.bluetooth_volume);
  const play = useProps(props => props.play);
  const switchState = useProps(props => props.switch);
  const search_list_track = useProps(props => props.search_list_track); // 查询的列表歌曲数据
  const list_select = useProps(props => props.list_select); // 列表选择 01 01 表示 存在 TF卡 和 USB
  const bluetooth_status = useProps(props => props.bluetooth_status);
  const bluetooth_info = useProps(props => props.bluetooth_info);

  const name = useDevice(device => device.devInfo.name);

  const musicList = useSelector(selectMusicList);
  const favorites = useSelector(selectFavorite);

  const [SVal, setSVal] = useState(0); // 音量
  const [showTips, setShowTips] = useState(false);

  useEffect(() => {
    setSVal(bluetooth_volume);
  }, [bluetooth_volume]);

  useEffect(() => {
    console.log('switchState', switchState);
  }, [switchState]);

  useEffect(() => {
    if (!list_select) return;
    const tf = parseInt(list_select.slice(0, 2), 16);
    const usb = parseInt(list_select.slice(2, 4), 16);
    tf && cardTypes.push({ id: 1, name: I18N.getLang('sd_card') });
    usb && cardTypes.push({ id: 2, name: 'USB' });
    dispath(updatedMusicTag(cardTypes));
  }, [list_select]);

  useEffect(() => {
    // const get = async () => {
    //   const res: any = await getStorage('favorites');
    //   res && dispath(updatedFavorites(res?.data.value || []));
    //   const params = cardTypes[card_index]?.id.toString(16).padStart(2, '0');
    //   if (!params) return;
    //   await dpaction.search_list_track.set(params);
    // };
    // get();
  }, []);

  useEffect(() => {
    bluetooth_status === 'disconnected' && setShowTips(true);
    bluetooth_status !== 'disconnected' && setShowTips(false);
    console.log('蓝牙状态', bluetooth_status);
  }, [bluetooth_status]);

  // useEffect(() => {
  //   if (!search_list_track) return;
  //   const num = parseInt(search_list_track.slice(2, 4), 16);
  //   total_num = parseInt(search_list_track.slice(0, 2), 16);
  //   if (search_list_track.length <= 4) {
  //     // 判断没有音乐列表
  //     total_num = 0;
  //     const newMusicList = _.cloneDeep(musicList);
  //     const name = cardTypes[card_index]?.name;
  //     newMusicList.forEach(item => {
  //       if (item.name === name) {
  //         item.musics = [];
  //         item.hex = '';
  //       }
  //     });
  //     dispath(updatedMusicList(newMusicList));
  //     return;
  //   }
  //   if (num !== cur_package_num) return;
  //   if (cur_package_num === 1) {
  //     // 第一包的数据
  //     const hex = search_list_track.slice(6);
  //     const name = cardTypes[card_index]?.name;
  //     const newMusicList = _.cloneDeep(musicList);
  //     const index = newMusicList.findIndex(item => item.name === name);
  //     index === -1
  //       ? newMusicList.push({ name, musics: [], hex: hex })
  //       : (newMusicList[index] = { name, musics: [], hex });
  //     dispath(updatedMusicList(newMusicList));
  //   } else {
  //     // 拼包数据
  //     const newMusicList = _.cloneDeep(musicList);
  //     const name = cardTypes[card_index]?.name;
  //     newMusicList.forEach(item => {
  //       if (item.name === name) {
  //         // 最后一个数组对象添加数据
  //         item.hex = item.hex + search_list_track.slice(6);
  //       }
  //     });
  //     dispath(updatedMusicList(newMusicList));
  //   }
  //   cur_package_num++;
  //   if (total_num !== num) return;
  //   total_num = 0;
  //   cur_package_num = 1;
  //   handleParseGB2312();
  //   if (card_index < cardTypes.length - 1) {
  //     ++card_index;
  //     dpaction.search_list_track.set(cardTypes[card_index].id.toString(16).padStart(2, '0'));
  //   }
  //   music_id = 0;
  // }, [search_list_track]);

  /**
   * 解析歌曲数据
   */
  // useEffect(() => {
  //   if (total_num !== 0 || cur_package_num !== 1) return;
  //   const music = musicList.find(item => item.musics.length === 0);
  //   if (!music) return;
  //   handleParseGB2312();
  //   if (card_index < cardTypes.length - 1) {
  //     ++card_index;
  //     dpaction.search_list_track.set(cardTypes[card_index].id.toString(16).padStart(2, '0'));
  //   }
  //   music_id = 0;
  // }, [musicList]);

  /**
   * 解析gb2312编码
   */
  const handleParseGB2312 = () => {
    const newMusicList = _.cloneDeep(musicList);
    newMusicList.forEach(item => {
      const favoriteSet = new Set(
        favorites.find(favorite => favorite.type === item.name)?.musics || []
      );
      const gb2312Buffer = Buffer.from(item.hex, 'hex');
      const utf8String = iconv.decode(gb2312Buffer, 'gb2312');
      const result = utf8String.split('\r\n').filter(line => line !== '');

      result.forEach(name => {
        if (name.startsWith('F:')) {
          item.musics.push({
            file: name.replace('F:', ''),
            data: [],
            open: true,
          });
        } else {
          if (!item.musics[item.musics.length - 1]) return;
          item.musics[item.musics.length - 1].data.push({
            name: name.replace('.mp3', ''),
            id: ++music_id,
            favorite: favoriteSet.has(music_id),
          });
        }
      });
    });
    dispath(updatedMusicList(newMusicList));
  };

  /**
   * 监听音量的变化
   * @param {number} val - 0~100
   */
  const handleSliderChange = (val: number) => {
    setSVal(val);
  };

  /**
   * 监听音量结束
   * @param {number} val - 0~100
   */
  const handleSliderTouchEnd = (val: number) => {
    setSVal(val);
    dpaction.bluetooth_volume.set(val, { throttle: 300 });
  };

  /**
   * 处理点击播放暂停
   */
  const handlePlay = () => {
    dpaction.play.toggle({ throttle: 300 });
  };

  return (
    <View className={styles.view}>
      <Navbar title={name} />
      <View
        style={{
          flex: '1',
          display: 'flex',
          flexDirection: 'column',
        }}
      >
        <BleState />
        {/* 内容区域组件 */}

        <View className={styles.sound}>
          <View style={{ display: 'flex', flexDirection: 'column', alignItems: 'center' }}>
            <View style={{ marginBottom: '14rpx', color: '#333333', fontSize: '28rpx' }}>
              {SVal + '%'}
            </View>
            <Foo
              min={0}
              max={100}
              value={bluetooth_volume}
              width="52rpx"
              height="600rpx"
              barColor="linear-gradient(0deg, #ea5613 0%, #f6cf14 100%)"
              trackStyle={{
                borderRadius: '100rpx',
                width: '52rpx',
                height: '600rpx',
                overflow: 'hidden',
                padding: '0',
                margin: '0',
              }}
              showIcon={false}
              showText={false}
              onChange={handleSliderChange}
              onTouchEnd={handleSliderTouchEnd}
            />
          </View>
          <View style={{ flex: 1, position: 'relative', pointerEvents: 'none' }}>
            <Image className={styles.logo} src={require('@/static/imgs/home/logo.png')} />
          </View>
        </View>
      </View>

      {/* 控制组件 */}
      <View className={styles.control}>
        <View
          className={styles.last}
          hoverClassName={styles.hover}
          onClick={() => {
            dpaction.change_control.set('last', { throttle: 300 });
          }}
        >
          <Image src={require('@/static/imgs/home/last_ic.png')} />
        </View>
        <View className={styles.play} hoverClassName={styles.hover} onClick={() => handlePlay()}>
          {!play ? (
            <Image src={require('@/static/imgs/home/play_ic.png')} />
          ) : (
            <Image src={require('@/static/imgs/home/stop_ic.png')} />
          )}
        </View>
        <View
          className={styles.next}
          hoverClassName={styles.hover}
          onClick={() => {
            dpaction.change_control.set('next', { throttle: 300 });
          }}
        >
          <Image src={require('@/static/imgs/home/next_ic.png')} />
        </View>
      </View>

      <Tabbar />

      {/* 关闭的蒙版层 */}
      {!switchState && <View className={styles.mask}></View>}

      {/* 连接蓝牙对话框组件区域 */}
      <Popup
        show={showTips}
        position="center"
        round
        customStyle={{
          width: '560rpx',
          boxSizing: 'border-box',
          backgroundImage: `url(${require('@/static/imgs/common/custom_picker_bg.png')})`,
          backgroundSize: 'cover',
        }}
      >
        <View className={styles.tipsPopup}>
          {/* 连接蓝牙提示: */}
          <View className={styles.label}>{I18N.getLang('connect_tips')}</View>
          {/* 是否连接蓝牙音频设备? */}
          <View className={styles.input}>
            {isIos ? I18N.getLang('connect_ios_content') : I18N.getLang('connect_content')}
          </View>
          <View className={styles.btns}>
            <View
              className={styles.cancal}
              onClick={() => {
                setShowTips(false);
              }}
            >
              {I18N.getLang('cancel')}
            </View>
            <View
              className={styles.confirm}
              onClick={() => {
                setShowTips(false);
                !isIos &&
                  connectBTBond({
                    mac: bluetooth_info,
                    fail: err => {
                      console.log('蓝牙配对失败', bluetooth_info, err);
                      openSystemBluetoothSetting();
                    },
                    complete: () => {
                      console.log('蓝牙mac地址', bluetooth_info);
                    },
                  });
              }}
            >
              {I18N.getLang('confirm')}
            </View>
          </View>
        </View>
      </Popup>
    </View>
  );
};

export default Home;
